#ifndef __UTILS__
#define __UTILS__

#include "CommonPatchers.h"
extern "C"
{
#include "ntifs.h"
}

#define STATUS_SUCCESS            0
#define STATUS_PATCH_NOT_FOUND    1
#define STATUS_INCORRECT_SIZE     2
#define STATUS_BAD_ALLOC          3
#define STATUS_INVALID_PARAMS     4   
#define STATUS_NOT_FOUND          5
#define STATUS_UNSUCCESSFUL       6
#define NOT_SUPPORTED             7

struct _MemoryManager;

typedef int (__stdcall *Patcher_type)(struct _MemoryManager* pThis, void * pBuffer, const void * pInfo, size_t size2Patch);
typedef int (__stdcall *UnProtector_type)(struct _MemoryManager* pThis, void * pBuffer);

typedef struct _MemoryManager
{
    UnProtector_type pUnProtect;
    Patcher_type pPatcher;
    void* pContext;
} MemoryManager;


typedef  void* (__stdcall *AllocateBufferProc)(struct _Allocator * pThis,
                                               int iSize);
typedef  void (__stdcall *FreeBufferProc)(struct _Allocator * pThis,
                                          void* pBuffer);

typedef struct _Allocator
{
    AllocateBufferProc pAllocateBuffer;
    FreeBufferProc pFreeBuffer;
    void* pContext;
}Allocator;

typedef struct _SimplePatchContext
{
    MemoryManager * pMemoryManager;
    Allocator * pAllocator;
    Allocator * pHookAllocator;
}SimplePatchContext;

NTSTATUS PatchNtosFunction_npaged(wchar_t * pFunctionName, 
                                     ULONG nameSizeInBytes, 
                                     void * pHook, 
                                     int * pRes);

static int UnProtect(MemoryManager* pAllocator, void * pBuffer)
{
    return 0;
}

PVOID GetKernelAnalogAddress(ULONG hModule,PCHAR FuncName);
ULONG GetModuleBase(PCHAR Mod);

typedef 
NTKERNELAPI
NTSTATUS
(*MmProtectMdlSystemAddressPtr)(
                                IN PMDL  MemoryDescriptorList,
                                IN ULONG  NewProtect 
                                );

static void * __stdcall AllocateBuffer_NonPaged(Allocator* pAllocator, int iSize)
{
    void * pData = ExAllocatePool(NonPagedPool, iSize);
    return pData;
}

static void __stdcall FreeBuffer_NonPaged(Allocator* pAllocator, void * pBuffer)
{
    ExFreePool(pBuffer);
}//-------------------------------------------------------------------------

Allocator g_kernelNonPagedAllocator = {AllocateBuffer_NonPaged, FreeBuffer_NonPaged};
MemoryManager g_kernelMemoryManager = {UnProtect, 0};

int __stdcall PatchSomething(SimplePatchContext * pContext, //IN
                                void * pPlaceToPatch,  // IN
                                void * pHook);         // IN

#define MAX_PATCH_SIZE                      16

typedef struct _PatchResult
{
    unsigned char * pChangedCode;
    int iPatchedSize;
} PatchResult;

typedef struct _FunctionRuntimeInfo
{
    void * pAddress;        // OUT (IN/OUT in PatchFunctions2)
    void * pHook;           // ALWAYS OUT 
    void * pHookHandler;    // IN in PatchFunctions2
    PatchResult result;   
    unsigned char resultBuffer[MAX_PATCH_SIZE];
}
TransactionInfo;

static struct Target * g_knownTargets[] =
{
    &g_STD_Target,
    &g_PushConstTarget,
    &g_STD2,
    &g_STD3,
    &g_STD4,
    &g_STD5,
    &g_STD6,
    &g_STD7,
    &g_STD8,
    &g_STD9,
    &g_STD10,
    &g_STD11,
    &g_STD12,
    &g_STD13,
    &g_STD14,
    &g_STD15,
    &g_STD16,
    0
};

struct _MemoryReader;
typedef  int (__stdcall *TMemoryReader_Ptr)  (  struct _MemoryReader * pThis,
                                              char* pPlace,
                                              char* pBuffer,
                                              unsigned long ulSize,
                                              unsigned long* pulSizeOut );


typedef struct _MemoryReader
{
    TMemoryReader_Ptr   pRead;
    void*               pContext;

} MemoryReader;

struct _MemoryWriter;
typedef  int (__stdcall *TMemoryWriter_Ptr)  (struct _MemoryWriter * pThis,
                                              char* pPlace,
                                              char* pBuffer,
                                              unsigned long ulSize,
                                              unsigned long* pulSizeOut );

typedef struct _MemoryWriter
{
    TMemoryWriter_Ptr   pWrite;
    void*               pContext;

} MemoryWriter;

typedef struct _MemoryRWManager
{
    MemoryReader * pReader;
    MemoryWriter * pWriter;
} MemoryRWManager;

int PatchFunctions2(MemoryManager * pMemoryManager,
                       Allocator * pAllocator,
                       Allocator * pHookAllocator,
                       struct Target ** ppTargetInfoVec,
                       TransactionInfo * pTransactionInfo, 
                       int iFunctionsCount);

int PatchFunctions2Ex(MemoryManager * pMemoryManager,
                         MemoryRWManager * pRwManager,
                         Allocator * pAllocator,
                         Allocator * pHookAllocator,
                         struct Target ** ppTargetInfoVec,
                         TransactionInfo * pTransactionInfo, 
                         int iFunctionsCount);

int TryPatchCodeEx(MemoryRWManager *pRwManager,
                      unsigned char * pAddress, 
                      MemoryManager * pMemoryManager,
                      struct Target ** targets,
                      void * pHook);

int PatchCodeEx(MemoryRWManager *pRwManager,
                      unsigned char * pAddress, 
                      MemoryManager * pMemoryManager,
                      struct Target ** targets, 
                      void * pHook,
                      int bNeedPatch,
                      PatchResult * pResult); // out

int __stdcall ReadMemoryWithReader(  MemoryReader* pMemoryReader,
                      char* pPlace,
                      char* pBuffer,
                      unsigned long ulSize,          
                      unsigned long* pulSizeOut );

int __stdcall PatchMemory(MemoryManager* pMemoryManager, 
                             void * pBuffer, 
                             const void * pInfo, 
                             size_t size2Patch);

int __stdcall UnprotectMemory(MemoryManager* pMemoryManager,
                                 void* pBuffer);

 MemoryRWManager * GetInternalMemoryRwManager();

#define MEMCPY(dest, src, size_in) \
 {   \
 char * pInData = (char * )(src); \
 char * pOutData = (char * )(dest);\
 size_t size = size_in;\
 for(size_t i = 0; i < size; ++i)\
 {\
 pOutData[i] = pInData[i];\
 }\
 }

 static int 
     __stdcall MemoryRead_Default(MemoryReader* fpThis,
     char* pPlace,
     char* pBuffer,
     unsigned long ulSize,
     unsigned long* pulSizeOut)
 {
     MEMCPY(pBuffer, pPlace, ulSize);
     if (pulSizeOut)
         *pulSizeOut = ulSize;
     return 0;
 }

 static int __stdcall MemoryWrite_Default(MemoryWriter* fpThis,
     char* pPlace,
     char* pBuffer,
     unsigned long ulSize,
     unsigned long* pulSizeOut)
 {
     MEMCPY(pPlace, pBuffer, ulSize);
     if (pulSizeOut)
         *pulSizeOut = ulSize;
     return 0;
 }

 static MemoryReader g_defaultMemoryReader = {MemoryRead_Default, 0};
 static MemoryWriter g_defaultMemoryWriter = {MemoryWrite_Default,0};
 static MemoryRWManager g_defaultRwManager = {&g_defaultMemoryReader, &g_defaultMemoryWriter};

 //-------------------------------------------------------------------------
 // AllocateHook
 //    function allocates hook and fills it by nops
 int __stdcall AllocateHookEx(MemoryRWManager * pRwManager,
     Allocator * pAllocator, 
     void ** pHook);
 //-------------------------------------------------------------------------
 // InitializeHook
 //    function puts [call pHandler] command into the pHook address
 int __stdcall InitializeHook(unsigned char * pHook, 
     unsigned char * pHandler);
 //-------------------------------------------------------------------------
 // InitializeHookEx
 //    function puts [call pHandler] command into the pHook address
 int __stdcall InitializeHookEx(MemoryRWManager * pRwManager,
     unsigned char * pHook, 
     unsigned char * pHandler);

 void* __stdcall AllocateMemory(Allocator* pAllocator,
     int iSize);

 void __stdcall FreeMemory(Allocator* pAllocator,
     void* pBuffer);

 int __stdcall AllocatePostParams(Allocator * pAllocator, // in
     int size,                  // in 
     void * pPostHandler,       // in
     void ** ppParamsData, // out
     void ** ppStub);      // out

#define FORM_POST_PARAMS(pParamsData, size, pPostHandler, pCaller)  \
    pCaller = pParamsData+size; \
    *pCaller = 0x68;  \
    *(void**)(pCaller+1) = pParamsData; \
    pCaller+=5;\
    *pCaller = 0xE8;\
    *(void**)(pCaller+1) = (void*)((unsigned char * )pPostHandler - pCaller - 5);\
    pCaller = (unsigned char * )(pParamsData+size);

#define GET_STUB_SIZE(sizeOfData) (sizeOfData + 10)

#define GET_PARAM(FirstParam, Number) \
    (*((unsigned long *)FirstParam + Number))

#define GET_PARAM_PTR(FirstParam, Number) \
    (((unsigned long *)FirstParam + Number))

 int __stdcall WriteMemoryWithWriter(MemoryWriter* pMemoryWriter,
     char* pPlace,
     char* pBuffer,
     unsigned long ulSize,          
     unsigned long* pulSizeOut);

 typedef NTSTATUS (*PZwProtectVirtualMemoryType)(
     IN HANDLE  ProcessHandle,
     IN OUT PVOID  *BaseAddress,
     IN OUT PULONG  ProtectSize,
     IN ULONG  NewProtect,
     OUT PULONG  OldProtect);

 //-----------------------------------------------
 void __stdcall InitKernelAllocators(PZwProtectVirtualMemoryType pZwProtectVirtualMemory,
     Patcher_type pPatcher);
 //-----------------------------------------------

 int Defence_Patcher(MemoryManager* pMemoryManager, 
     void * pBuffer, 
     const void * pInfo, 
     size_t size2Patch);

#endif //__UTILS__
